;
; FILETTATURA.LSP
; 1/1/2001 Aniello Annunziata per CADLandia
;
; Questo lisp crea una filettatura (sia maschio che femmina) modellando delle superfici
; il risultato va benissimo per uso "estetico" ma non  un solido modificabile.
; Altre limitazioni del programma sono che ogni spira  uan entit singola.
; Questo  dovuto al fatto che lo strumento 3DMESH di AutoCAD pu costruire mesh di massimo
; 256x256. Poich la risoluzione consigliata per ogni spira  di minimo 16, ci significava
; che non si potevano costruire reti da pi di 8 spire (32x4x4).
; Per ovviare a questo problema abbiamo dovuto creare entit distinte per ogni spira.
; Questo ci ha comportato, poi, non pochi problemi in fase di programmazione, che ci hanno
; fatto ricorrere ad alcuni artifizi.
;
; Ogni spira  una 3DMESH di (n+1)x4 punti, questo perch per fare un giro completo
; se la risoluzione  n, ci vogliono n+1 punti.
; Allo stesso modo, per fare un triangolo chiuso estruso, ci vogliono 3+1 punti (cio
; bisogna ritornare nel punto origine).
;

; Funzione principale

(defun C:FILETTATURA ()
;Salvo i valori delle variabili di sistema che modificher.
  (setq oldcmd (getvar "cmdecho")
	oldblp (getvar "blipmode")
	oldosm (getvar "osmode")
  ); setq
  (setvar "cmdecho" 0)
;Acquisisco i valori dagli imput degli utenti
  (initget 1)	;non accetto valori nulli
  (setq cn (getpoint "\nCentro della filettatura: "))
  (initget 1)	;non accetto valori nulli
  (setq ag0 (getangle cn "\nAngolo di rotazione dell'inizio della filettatura: "))
  (initget 7)	;non accetto valori nulli, ne lo zero, ne negativi
  (setq r1 (getdist cn "\nRaggio della filettatura (in mm): "))
  (initget 7)	;non accetto valori nulli, ne lo zero, ne negativi
  (setq ng (getint "\nNumero di spire: "))
  (initget 7)	;non accetto valori nulli, ne lo zero, ne negativi
  (setq hf (getreal "\nAltezza filetto (in mm): "))
  (initget 3)	;non accetto valori nulli o zero.
  (setq sf (getreal "\nSpessore filetto (in mm) (+ filetto maschio) (- filetto femmina): "))
  (initget 7)	;non accetto valori nulli, ne lo zero, ne negativi
  (setq nr (getint "\nRisoluzione del filetto (per giro): "))
  (setq dht  (/ hf nr))		;calcolo il deltah
  (setq dag  (/ (* 2 pi) nr))	;calcolo il passo angolare
  (setq r2 (+ r1 sf))		;calcolo il raggio della punta del filetto
  (setq ag1 ag0)		;calcolo l'angolo inziale
  (setq tyn 4)			;numero di elementi lungo la direzione M della mesh
  (setvar "osmode" 0)
  (setvar "blipmode" 0)
  (setq contg 0)		;resetto il contatore delle spire
;Calcolo dei punti
;Ripeto per ogni spira
  (repeat ng
;Resetto la lista dei punti ed il contatore dei punti per spira
    (setq plist nil)
    (setq count 0)
;Uso un artifizio per ritornare nell'angolo giusto quando creo una spira che non 
;la prima (in effetti la spira deve iniziare l dove  finita la precedente, per cui devo
;tornare indietro di uno step angolare).
    (if (> contg 0)
      (setq ag1 (- ag1 dag))
      ); if
;Ripeto per ogni punto di risoluzione
    (repeat (+ nr 1)
;Calcola i punti nel piano attraverso la funzione polar
      (setq p1 (polar cn ag1 r1)	;punto "interno"
	    p2 (polar cn ag1 r2)	;punto "esterno"
	    ); setq
;Calcolo le quote dei punti
      (setq numpunto (+ count (* contg nr)))	;mantiene il numero progressivo del punto
      (setq pa (sp p1 (* numpunto dht))		;assegna la quota al punto "interno" basso
	    pb (sp p2 (+ (* numpunto dht) (/ hf 2)))	;assegna la quota al punto "esterno" 
	    pc (sp pa hf)			;assegna la quota al punto "interno" alto
	    ); setq
;Accodo alla lista dei punti da passare al comando 3DMESH
      (setq plist (append plist (list pa pb pc pa)))
;Mi muovo angolarmente di uno step ed aggiorno il contatore dei punti nella spira
      (setq ag1 (+ ag1 dag)
	    count (+ 1 count)
	    );setq
    ); repeat (per ogni punto)
;Calcolo il numero di punti nella lista
      (setq np (length plist))
;Disegno la 3D mesh, dando l'esatto numero di punti in direzione N ed in direzione M
      (command "_3dmesh" (/ np tyn) tyn)
;Fornisco i punti al comando 3DMESH
      (repeat np
        (command (car plist))
        (setq plist (cdr plist))
      ); repeat
;Aggiorno il contatore delle spire
      (setq contg (+ contg 1))
  ); repeat (per ogni spira)
;Ripristino lo stato delle variabili di sistema modificate
  (setvar "osmode" oldosm)
  (setvar "blipmode" oldblp)
  (setvar "cmdecho" oldcmd)
  (princ)
)
;
;--------------------------------------------------------------------
;Ottieni i punti incrementati dell'altezza del punto in direzione Z
;--------------------------------------------------------------------
(defun sp (pt h)
  (list (car pt)(cadr pt)(+ (caddr pt) h))
)
;
;
;--------------------------------------------------------------------
;Messaggio stampato su caricamento
;--------------------------------------------------------------------
;
(princ "\nFILETTATURA.LSP caricato. Digita FILETTATURA per disegnare filettatura.")
(princ)